#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <limits.h>

int main
(
 int argc,
 char **argv
 )
{
  long nl;
  int n;
  double *data_p = NULL;
  int index;
  int fscanf_rc;
  float datum;

  int stats_rc;
  int quart_rc;
  double mean;
  double variance;
  double lower;
  double median;
  double upper;

  /*
   * 1. Parse argument
   * 2. Allocate space
   * 3. Read data
   * 4. Process data
   * 5. Print results
   */


  /* 
   * 1. Argument parsing
   */

  if (2 != argc)
    {
      fprintf(stderr, "Usage: %s num\n", argv[0]);
      exit(1);
    }

  nl = strtol(argv[1], (char **)NULL, 10);
  if (0 != errno)
    {
      fprintf(stderr, "%s: Cannot parse argument \"%s\": %s\n", argv[0], argv[1], strerror(errno));
      exit(1);
    }

  if ((0 >= nl) || (INT_MAX <= nl))
    {
      fprintf(stderr, "%s: Argument (\"%s\") must be a strictly positive integer less than %d.\n", argv[0], argv[1], INT_MAX);
      exit(1);
    }

  n = (int)nl;


  /* 
   * 2. Space allocations
   */

  data_p = (double *)calloc(n, sizeof(double));


  /* 
   * 3. Read in data from stdin
   */
  for (index=0; index<n; index+=1)
    {
      fscanf_rc = fscanf(stdin, "%f\n", &datum);
      if (1 != fscanf_rc)
	{
	  fprintf(stderr, "%s: Could not read input line %d\n", argv[0], index+1);
	  exit(1);
	}
      data_p[index] = datum;
    }


  /* 
   * 4. Now do something with the data
   */
  stats_rc = stats(data_p, n, &mean, &variance);
  if (0 == stats_rc)
    {
      fprintf(stdout, "Mean:\t\t%f\nVariance:\t%f\n", mean, variance);
    }
  else
    {
      fprintf(stderr, "%s: stats() function failed.\n", argv[0]);
    }

  quart_rc = quartiles(data_p, n, &lower, &median, &upper);
  if (0 == quart_rc)
    {
      fprintf(stdout, "Lower quartile:\t%f\nMedian:\t\t%f\nUpper quartile:\t%f\n", lower, median, upper);
    }
  else
    {
      fprintf(stderr, "%s: quart() function failed.\n", argv[0]);
    }
  
  exit(0);
}
