#include <stdio.h>
#include <limits.h>

double double_abs
(
 double x
)
{
  if (x < 0.0)
  {
    return -1.0*x;
  }
  else
  {
    return x;
  }

}

/* 
 * Warning: if either number is or both numbers are a NaN then the two
 * numbers are declared equal.
 */
int abs_compare
(
 double *a_p,
 double *b_p
 )
{
  if (double_abs(*a_p) > double_abs(*b_p))
    {
      return 1;
    }
  else if (double_abs(*a_p) < double_abs(*b_p))
    {
      return -1;
    }
  else
    {
      return 0;
    }
}




int stats
(
 double *data_p,
 int n,
 double *mean_p,
 double *variance_p
 )
{
  /*
   * 1. Check inputs for validity
   * 2. Sort data
   * 3. Calculate mean
   * 4. Calculate variance
   * 5. Assign values
   * 6. Return
   */

  double sum=0.0;
  double dev=0.0;
  double mean;
  double variance;
  int index;

  /* 1. Check inputs */
  if (NULL == data_p)
    {
      fprintf(stderr, "stats(): Array is NULL\n");
      return 1;
    }

  if ((0>=n) || (INT_MAX<=n))
    {
      fprintf(stderr, "stats(): Array size (%d) must be strictly positive and strictly less than %d\n", n, INT_MAX);
      return 1;
    }

  if (NULL == mean_p)
    {
      fprintf(stderr, "stats(): Pointer to mean value is NULL\n");
      return 1;
    }

  if (NULL == variance_p)
    {
      fprintf(stderr, "stats(): Pointer to variance value is NULL\n");
      return 1;
    }

  /* 2. Sort data */
  qsort(data_p, n, sizeof(double), abs_compare);


  /* 3. Mean */
  for (index=0; index<n; index+=1)
    {
      sum += data_p[index];
    }
  mean = sum/(double)(n);

  
  /* 4. Variance */
  for (index=n-1; index>=0; index-=1)
    {
      dev += (mean - data_p[index])*(mean - data_p[index]);
    }
  variance = dev/(double)(n);


  /* 5. Assign values */
  /* fprintf(stderr, "DEBUG: stat(): mean=%f variance=%f\n", mean, variance); */
  *mean_p = mean;
  *variance_p = variance;


  /* 6. Return */
  return 0;
  
}
