#include <stdio.h>
#include <limits.h>

/* 
 * Warning: if either number is or both numbers are a NaN then the two
 * numbers are declared equal.
 */
int compare
(
 double *a_p,
 double *b_p
 )
{
  if (*a_p > *b_p)
    {
      return 1;
    }
  else if (*a_p < *b_p)
    {
      return -1;
    }
  else
    {
      return 0;
    }
}





int quartiles
(
 double *data_p,
 int n,
 double *lower_p,
 double *median_p,
 double *upper_p
 )
{
  /*
   * 1. Check inputs for validity
   * 2. Sort data
   * 3. Median
   * 4. Lower quartile
   * 5. Upper quartile
   * 6. Return
   */

  int offset;
  int remain;

  /* 1. Check inputs */
  if (NULL == data_p)
    {
      fprintf(stderr, "stats(): Array is NULL\n");
      return 1;
    }

  if ((0>=n) || (INT_MAX<=n))
    {
      fprintf(stderr, "stats(): Array size (%d) must be strictly positive and strictly less than %d\n", n, INT_MAX);
      return 1;
    }

  if (NULL == lower_p)
    {
      fprintf(stderr, "stats(): Pointer to lower quartile value is NULL\n");
      return 1;
    }

  if (NULL == median_p)
    {
      fprintf(stderr, "stats(): Pointer to median value is NULL\n");
      return 1;
    }

  if (NULL == upper_p)
    {
      fprintf(stderr, "stats(): Pointer to upper quartile value is NULL\n");
      return 1;
    }




  /* 2. Sort data */
  qsort(data_p, n, sizeof(double), compare);


  /* 3. Median */
  offset = (n-1)/2;
  remain = (n-1)%2;
  if (0 == remain)
    {
      *median_p = data_p[offset];

    }
  else if (1 == remain)
    {
      *median_p = (data_p[offset] + data_p[offset+1])/2.0;
    }
  else
    {
      fprintf(stderr, "quartiles(): Bogus value for median remainder: %d\n", remain);
      return 1;
    }


  /* 4. Lower quartile */
  offset = (n-1)/4;
  remain = (n-1)%4;
  if (0 == remain)
    {
      *lower_p = data_p[offset];
    }
  else if (1 == remain)
    {
      *lower_p = (3.0*data_p[offset] +     data_p[offset+1])/4.0;
    }
  else if (2 == remain)
    {
      *lower_p = (    data_p[offset] +     data_p[offset+1])/2.0;
    }
  else if (3 == remain)
    {
      *lower_p = (    data_p[offset] + 3.0*data_p[offset+1])/4.0;
    }
  else
    {
      fprintf(stderr, "quartiles(): Bogus value for lower quartile remainder: %d\n", remain);
      return 1;
    }


  /* 5. Upper quartile */
  offset = (3*n-3)/4;
  remain = (n-1)%4;
  if (0 == remain)
    {
      *upper_p = data_p[offset];
    }
  else if (1 == remain)
    {
      *upper_p = (    data_p[offset] + 3.0*data_p[offset+1])/4.0;
    }
  else if (2 == remain)
    {
      *upper_p = (    data_p[offset] +     data_p[offset+1])/2.0;
    }
  else if (3 == remain)
    {
      *upper_p = (3.0*data_p[offset] +     data_p[offset+1])/4.0;
    }



  /* 6. Return */
  return 0;
  
}
