#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include <sys/errno.h>
#include <math.h>

/*
  This program iterates the unit square under the iteration
  x' = x + y
  y' = y * ( 1 + epsilon*sin(2pi*x) )
  with both calculations modulo 1.0.

  The system iterates a series of points whose initial positions form
  an nx by ny regular grid on the unit square.  Each point is iterated
  "n_iter" many times.

 */

int main
(
 int argc,
 char **argv
 )
{
  const int nx = 100;
  const int ny = 100;
  const int n_iter = 100;

  double epsilon = 0.0;

  const double twopi = 2.0*3.14159265358979323846;

  int ix = 0;
  int iy = 0;
  double x = 0.0;
  double y = 0.0;
  double xx = 0.0;
  double yy = 0.0;
  int count = 0;
  double junk = 0.0; /* Used to discard integer part in modf() */

  /* Process the user's command line arguments. */
  if (argc != 2)
    {
      fprintf(stderr, "Wrong number of arguments!\n1 expected, %d found.\nUsage: %s epsilon\n", (argc-1), argv[0]);
      exit(1);
    }


  epsilon = strtod(argv[1], NULL);
  if (0 != errno)
    {
      perror("Invalid epsilon: ");
      exit(1);
    }

  /* Run */
  for (ix = 0L; ix < nx; ix += 1)
    {
      for (iy = 0L; iy < ny; iy += 1)
	{
	  x = ((double)ix)/((double)nx);
	  y = ((double)iy)/((double)ny);
	  for (count=0; count < n_iter; count += 1)
	    {
	      xx = modf(1.0 + x + y, &junk);
	      yy = modf(1.0 + y + epsilon*y*sin(twopi*x), &junk);
	      x = xx;
	      y = yy;
	    }
	  printf("%f\t%f\n", x, y);
	}
    }

  /* All done! */
  exit(0);
}
